@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 Tigera, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Tigera, Inc. All rights reserved.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -31,7 +31,6 @@ import (
 	"github.com/projectcalico/api/pkg/lib/numorstring"
 	log "github.com/sirupsen/logrus"
 
-	"github.com/projectcalico/calico/confd/pkg/buildinfo"
 	"github.com/projectcalico/calico/confd/pkg/config"
 	logutils "github.com/projectcalico/calico/confd/pkg/log"
 	"github.com/projectcalico/calico/confd/pkg/resource/template"
@@ -47,6 +46,7 @@ import (
 	cnet "github.com/projectcalico/calico/libcalico-go/lib/net"
 	"github.com/projectcalico/calico/libcalico-go/lib/options"
 	"github.com/projectcalico/calico/libcalico-go/lib/set"
+	"github.com/projectcalico/calico/pkg/buildinfo"
 	"github.com/projectcalico/calico/typha/pkg/syncclientutils"
 	"github.com/projectcalico/calico/typha/pkg/syncproto"
 )
@@ -158,6 +158,9 @@ func NewCalicoClient(confdConfig *config.Config) (*client, error) {
 		ClusterIPRouteIndex:      NewRouteIndex(),
 		LoadBalancerIPRouteIndex: NewRouteIndex(),
 
+		// Initialize service load balancer aggregation to enabled by default
+		serviceLoadBalancerAggregation: apiv3.ServiceLoadBalancerAggregationEnabled,
+
 		// This channel, for the syncer calling OnUpdates and OnStatusUpdated, has 0
 		// capacity so that the caller blocks in the same way as it did before when its
 		// calls were processed synchronously.
@@ -247,7 +250,7 @@ func NewCalicoClient(confdConfig *config.Config) (*client, error) {
 	c.nodeV1Processor = updateprocessors.NewBGPNodeUpdateProcessor(clientCfg.Spec.K8sUsePodCIDR)
 	if syncclientutils.MustStartSyncerClientIfTyphaConfigured(
 		&confdConfig.Typha, syncproto.SyncerTypeBGP,
-		buildinfo.GitVersion, template.NodeName, fmt.Sprintf("confd %s", buildinfo.GitVersion),
+		buildinfo.Version, template.NodeName, fmt.Sprintf("confd %s", buildinfo.Version),
 		c,
 	) {
 		log.Debug("Using typha syncclient")
@@ -380,6 +383,9 @@ type client struct {
 
 	// Cached value of the default BGP configuration for node to node mesh BGP password lookup.
 	globalBGPConfig *apiv3.BGPConfiguration
+
+	// Service load balancer aggregation setting
+	serviceLoadBalancerAggregation apiv3.ServiceLoadBalancerAggregation
 }
 
 // SetPrefixes is called from confd to notify this client of the full set of prefixes that will
@@ -481,6 +487,7 @@ func (c *client) inSync() bool {
 type bgpPeer struct {
 	PeerIP          cnet.IP              `json:"ip"`
 	ASNum           numorstring.ASNumber `json:"as_num,string"`
+	LocalASNum      numorstring.ASNumber `json:"local_as_num,string"`
 	RRClusterID     string               `json:"rr_cluster_id"`
 	Password        *string              `json:"password"`
 	SourceAddr      string               `json:"source_addr"`
@@ -650,10 +657,16 @@ func (c *client) updatePeersV1() {
 					reachableBy = v3res.Spec.ReachableBy
 				}
 
+				var localASN numorstring.ASNumber
+				if v3res.Spec.LocalASNumber != nil {
+					localASN = *v3res.Spec.LocalASNumber
+				}
+
 				keepOriginalNextHop, nextHopMode := getNextHopMode(v3res)
 				peers = append(peers, &bgpPeer{
 					PeerIP:          *ip,
 					ASNum:           v3res.Spec.ASNumber,
+					LocalASNum:      localASN,
 					SourceAddr:      string(v3res.Spec.SourceAddress),
 					Port:            port,
 					KeepNextHop:     keepOriginalNextHop,
@@ -884,6 +897,9 @@ func (c *client) localBGPPeerDataAsBGPPeers(localBGPPeerData localBGPPeerData, v
 			log.Warningf("Couldn't parse %v %v for workload %v", version, ipStr, workloadName)
 			continue
 		}
+		if v3Peer.Spec.LocalASNumber != nil {
+			peer.LocalASNum = *v3Peer.Spec.LocalASNumber
+		}
 		peer.PeerIP = *ip
 		peer.Filters = v3Peer.Spec.Filters
 		peer.ASNum = v3Peer.Spec.ASNumber
@@ -949,6 +965,7 @@ func (c *client) nodeAsBGPPeers(nodeName string, v4 bool, v6 bool, v3Peer *apiv3
 				log.WithError(err).Warningf("Problem parsing global AS number %v for node %v", asNum, nodeName)
 			}
 		}
+
 		peer.RRClusterID = rrClusterID
 
 		keepOriginalNextHop, nextHopMode := getNextHopMode(v3Peer)
@@ -1122,7 +1139,7 @@ func (c *client) onUpdates(updates []api.Update, needUpdatePeersV1 bool) {
 	for _, u := range updates {
 		if v3key, ok := u.Key.(model.ResourceKey); ok && v3key.Kind == apiv3.KindBGPConfiguration {
 			// Convert v3 BGPConfiguration to equivalent v1 cache values
-			v3res, _ := u.KVPair.Value.(*apiv3.BGPConfiguration)
+			v3res, _ := u.Value.(*apiv3.BGPConfiguration)
 			c.updateBGPConfigCache(v3key.Name, v3res, &needServiceAdvertisementUpdates, &needUpdatePeersV1, &needUpdatePeersReasons)
 		}
 		c.updateCache(u.UpdateType, &u.KVPair)
@@ -1217,6 +1234,14 @@ func (c *client) updateBGPConfigCache(resName string, v3res *apiv3.BGPConfigurat
 		c.getNodeMeshPasswordKVPair(v3res, model.GlobalBGPConfigKey{})
 		c.getIgnoredInterfacesKVPair(v3res, model.GlobalBGPConfigKey{})
 
+		// Update service load balancer aggregation setting
+		if v3res != nil && v3res.Spec.ServiceLoadBalancerAggregation != nil {
+			c.serviceLoadBalancerAggregation = *v3res.Spec.ServiceLoadBalancerAggregation
+		} else {
+			// Default to enabled if not specified or if v3res is nil
+			c.serviceLoadBalancerAggregation = apiv3.ServiceLoadBalancerAggregationEnabled
+		}
+
 		// Cache the updated BGP configuration
 		c.globalBGPConfig = v3res
 	} else if strings.HasPrefix(resName, perNodeConfigNamePrefix) {
@@ -1470,7 +1495,7 @@ func (c *client) getNodeMeshRestartTimeKVPair(v3res *apiv3.BGPConfiguration, key
 
 	if v3res != nil && v3res.Spec.NodeMeshMaxRestartTime != nil {
 		restartTime := *v3res.Spec.NodeMeshMaxRestartTime
-		c.updateCache(api.UpdateTypeKVUpdated, getKVPair(meshRestartKey, fmt.Sprintf("%v", int(math.Round(restartTime.Duration.Seconds())))))
+		c.updateCache(api.UpdateTypeKVUpdated, getKVPair(meshRestartKey, fmt.Sprintf("%v", int(math.Round(restartTime.Seconds())))))
 	} else {
 		c.updateCache(api.UpdateTypeKVDeleted, getKVPair(meshRestartKey))
 	}
@@ -1543,7 +1568,19 @@ func (c *client) onLoadBalancerIPsUpdate(lbIPs []string) {
 	// However, we don't want to advertise single IPs in this way because it breaks any "local" type services the user creates,
 	// which should instead be advertised from only a subset of nodes.
 	// So, we handle advertisement of any single-addresses found in the config on a per-service basis from within the routeGenerator.
-	globalLbIPs := filterNonSingleIPsFromCIDRs(lbIPs)
+
+	var globalLbIPs []string
+	if c.shouldAggregateLoadBalancerServicesLockHeld() {
+		// When service load balancer aggregation is enabled, we advertise the full CIDR ranges globally.
+		globalLbIPs = filterNonSingleIPsFromCIDRs(lbIPs)
+		log.Debug("Service load balancer aggregation enabled - advertising LoadBalancer CIDR ranges globally")
+	} else {
+		// When aggregation is disabled, we don't advertise any CIDR ranges globally.
+		// Instead, individual /32 routes will be advertised per service based on their LoadBalancer.Ingress.IP.
+		globalLbIPs = []string{}
+		log.Debug("Service load balancer aggregation disabled - not advertising LoadBalancer CIDR ranges globally")
+	}
+
 	if err := c.updateGlobalRoutes(globalLbIPs, c.LoadBalancerIPRouteIndex); err == nil {
 		c.loadBalancerIPs = lbIPs
 		c.loadBalancerIPNets = parseIPNets(c.loadBalancerIPs)
@@ -1571,6 +1608,18 @@ func (c *client) GetLoadBalancerIPs() []*net.IPNet {
 	return c.loadBalancerIPNets
 }
 
+// ShouldAggregateLoadBalancerServices ServiceLoadBalancerAggregationEnabled returns whether service load balancer aggregation is enabled
+func (c *client) ShouldAggregateLoadBalancerServices() bool {
+	c.cacheLock.Lock()
+	defer c.cacheLock.Unlock()
+	return c.shouldAggregateLoadBalancerServicesLockHeld()
+}
+
+// shouldAggregateLoadBalancerServicesLockHeld is an internal method that assumes the lock is already held
+func (c *client) shouldAggregateLoadBalancerServicesLockHeld() bool {
+	return c.serviceLoadBalancerAggregation == apiv3.ServiceLoadBalancerAggregationEnabled
+}
+
 // updateGlobalRoutes updates programs and withdraws routes based on the given CIDRs as provided via
 // the BGPConfiguration API, and this node's service advertisement status as configured via
 // the per-node Service advertisement exclusion label.
@@ -1964,7 +2013,7 @@ func (c *client) setPeerConfigFieldsFromV3Resource(peers []*bgpPeer, v3res *apiv
 		peer.Password = password
 		peer.SourceAddr = withDefault(string(v3res.Spec.SourceAddress), string(apiv3.SourceAddressUseNodeIP))
 		if v3res.Spec.MaxRestartTime != nil {
-			peer.RestartTime = fmt.Sprintf("%v", int(math.Round(v3res.Spec.MaxRestartTime.Duration.Seconds())))
+			peer.RestartTime = fmt.Sprintf("%v", int(math.Round(v3res.Spec.MaxRestartTime.Seconds())))
 		}
 	}
 }
