@@ -12,7 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-package ut
+package ut_test
 
 import (
 	"encoding/binary"
@@ -29,6 +29,7 @@ import (
 	"time"
 
 	. "github.com/onsi/gomega"
+	apiv3 "github.com/projectcalico/api/pkg/apis/projectcalico/v3"
 	log "github.com/sirupsen/logrus"
 
 	"github.com/projectcalico/calico/felix/bpf"
@@ -39,6 +40,7 @@ import (
 	"github.com/projectcalico/calico/felix/bpf/jump"
 	"github.com/projectcalico/calico/felix/bpf/maps"
 	"github.com/projectcalico/calico/felix/bpf/nat"
+	"github.com/projectcalico/calico/felix/bpf/qos"
 	"github.com/projectcalico/calico/felix/bpf/tc"
 	tcdefs "github.com/projectcalico/calico/felix/bpf/tc/defs"
 	"github.com/projectcalico/calico/felix/calc"
@@ -137,9 +139,9 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		err = bpfEpMgr.CompleteDeferredWork()
 		Expect(err).NotTo(HaveOccurred())
 
-		programsCount := 13
+		programsCount := 14
 		if ipv6Enabled {
-			programsCount = 25
+			programsCount = 27
 		}
 		Expect(programs.Count()).To(Equal(programsCount))
 		at := programs.Programs()
@@ -193,7 +195,9 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 			bpfEpMgr.OnUpdate(&proto.HostMetadataV6Update{Hostname: "uthost", Ipv6Addr: "1::4"})
 			err = bpfEpMgr.CompleteDeferredWork()
 			Expect(err).NotTo(HaveOccurred())
-			Expect(programs.Count()).To(Equal(50))
+			Expect(programs.Count()).To(Equal(52))
+
+			at := programs.Programs()
 
 			Expect(at).To(HaveKey(hook.AttachType{
 				Hook:       hook.Ingress,
@@ -331,10 +335,10 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		err := bpfEpMgr.CompleteDeferredWork()
 		Expect(err).NotTo(HaveOccurred())
 
-		programCount := 13
+		programCount := 14
 		jumpMapLen := 2
 		if ipv6Enabled {
-			programCount = 50
+			programCount = 52
 			jumpMapLen = 8
 		}
 		Expect(programs.Count()).To(Equal(programCount))
@@ -352,9 +356,9 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		err = bpfEpMgr.CompleteDeferredWork()
 		Expect(err).NotTo(HaveOccurred())
 
-		programsCount := 25
+		programsCount := 27
 		if ipv6Enabled {
-			programsCount = 50
+			programsCount = 52
 		}
 		Expect(programs.Count()).To(Equal(programsCount))
 
@@ -422,11 +426,11 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		err := bpfEpMgr.CompleteDeferredWork()
 		Expect(err).NotTo(HaveOccurred())
 
-		programsCount := 25
+		programsCount := 27
 		jumpMapLen := 6
 
 		if ipv6Enabled {
-			programsCount = 50
+			programsCount = 52
 			jumpMapLen = 16
 		}
 		Expect(programs.Count()).To(Equal(programsCount))
@@ -543,7 +547,7 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		Expect(attached).To(HaveKey("hostep2"))
 		Expect(attached).NotTo(HaveKey("workloadep3"))
 
-		programs.ResetCount() // Because we recycle it, restarted Felix would get a fresh copy.
+		programs.ResetForTesting() // Because we recycle it, restarted Felix would get a fresh copy.
 
 		bpfEpMgr, err = newBPFTestEpMgr(
 			&linux.Config{
@@ -587,10 +591,10 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		err = oldProgs.Open()
 		Expect(err).NotTo(HaveOccurred())
 		pm := jumpMapDump(oldProgs)
-		programsCount := 25
+		programsCount := 27
 		oldPoliciesCount := 4
 		if ipv6Enabled {
-			programsCount = 50
+			programsCount = 52
 			oldPoliciesCount = 12
 		}
 		Expect(pm).To(HaveLen(programsCount))
@@ -626,9 +630,9 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 		err = bpfEpMgr.CompleteDeferredWork()
 		Expect(err).NotTo(HaveOccurred())
 
-		Expect(programs.Count()).To(Equal(25))
+		Expect(programs.Count()).To(Equal(27))
 		pm = jumpMapDump(commonMaps.ProgramsMap)
-		Expect(pm).To(HaveLen(25))
+		Expect(pm).To(HaveLen(27))
 
 		pm = jumpMapDump(commonMaps.JumpMap)
 		// We remember the state from above
@@ -669,7 +673,7 @@ func runAttachTest(t *testing.T, ipv6Enabled bool) {
 
 		deleteLink(workload3)
 
-		programs.ResetCount() // Because we recycle it, restarted Felix would get a fresh copy.
+		programs.ResetForTesting() // Because we recycle it, restarted Felix would get a fresh copy.
 
 		bpfEpMgr, err = newBPFTestEpMgr(
 			&linux.Config{
@@ -759,15 +763,44 @@ func TestAttachWithMultipleWorkloadUpdate(t *testing.T) {
 			WorkloadId:     "workloadep1",
 			EndpointId:     "workloadep1",
 		},
-		Endpoint: &proto.WorkloadEndpoint{Name: "workloadep1"},
+		Endpoint: &proto.WorkloadEndpoint{
+			Name:        "workloadep1",
+			QosControls: &proto.QoSControls{IngressPacketRate: 50, IngressPacketBurst: 100, EgressPacketRate: 200, EgressPacketBurst: 300},
+		},
 	})
 	err = bpfEpMgr.CompleteDeferredWork()
 	Expect(err).NotTo(HaveOccurred())
 
-	valid, firstPrio, firstHandle := bpfEpMgr.GetIfaceQDiscInfo("workloadep1")
-	Expect(valid).To(BeTrue())
-	Expect(firstPrio).To(BeNumerically(">", 0))
-	Expect(firstHandle).To(BeNumerically(">", 0))
+	ingressProg, err := tc.ListAttachedPrograms("workloadep1", hook.Ingress.String(), true)
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(ingressProg)).To(Equal(1))
+
+	egressProg, err := tc.ListAttachedPrograms("workloadep1", hook.Egress.String(), true)
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(egressProg)).To(Equal(1))
+
+	Expect(ingressProg[0].Pref).To(Equal(egressProg[0].Pref))
+	Expect(ingressProg[0].Handle).To(Equal(egressProg[0].Handle))
+
+	// Verify that QoS map state is correctly created
+	qosMap := commonMaps.QoSMap
+	qosKey1 := qos.NewKey(uint32(workload1.Attrs().Index), 1)
+	qosValBytes1, err := qosMap.Get(qosKey1.AsBytes())
+	Expect(err).NotTo(HaveOccurred())
+	qosVal1 := qos.ValueFromBytes(qosValBytes1)
+	Expect(qosVal1.PacketRate()).To(Equal(int16(50)))
+	Expect(qosVal1.PacketBurst()).To(Equal(int16(100)))
+	Expect(qosVal1.PacketRateTokens()).To(Equal(int16(-1)))
+	Expect(qosVal1.PacketRateLastUpdate()).To(Equal(uint64(0)))
+
+	qosKey2 := qos.NewKey(uint32(workload1.Attrs().Index), 0)
+	qosValBytes2, err := qosMap.Get(qosKey2.AsBytes())
+	Expect(err).NotTo(HaveOccurred())
+	qosVal2 := qos.ValueFromBytes(qosValBytes2)
+	Expect(qosVal2.PacketRate()).To(Equal(int16(200)))
+	Expect(qosVal2.PacketBurst()).To(Equal(int16(300)))
+	Expect(qosVal2.PacketRateTokens()).To(Equal(int16(-1)))
+	Expect(qosVal2.PacketRateLastUpdate()).To(Equal(uint64(0)))
 
 	at := programs.Programs()
 	Expect(at).To(HaveKey(hook.AttachType{
@@ -797,16 +830,45 @@ func TestAttachWithMultipleWorkloadUpdate(t *testing.T) {
 				WorkloadId:     "workloadep1",
 				EndpointId:     "workloadep1",
 			},
-			Endpoint: &proto.WorkloadEndpoint{Name: "workloadep1"},
+			Endpoint: &proto.WorkloadEndpoint{
+				Name:        "workloadep1",
+				QosControls: &proto.QoSControls{IngressPacketRate: 50, IngressPacketBurst: 100, EgressPacketRate: 200, EgressPacketBurst: 300},
+			},
 		})
 		err = bpfEpMgr.CompleteDeferredWork()
 		Expect(err).NotTo(HaveOccurred())
-		valid, prio, handle := bpfEpMgr.GetIfaceQDiscInfo("workloadep1")
-		Expect(valid).To(BeTrue())
-		Expect(prio).To(Equal(firstPrio))
-		Expect(handle).To(Equal(firstHandle))
 	}
+	ingProg, err := tc.ListAttachedPrograms("workloadep1", hook.Ingress.String(), true)
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(ingProg)).To(Equal(1))
 
+	egrProg, err := tc.ListAttachedPrograms("workloadep1", hook.Egress.String(), true)
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(egrProg)).To(Equal(1))
+
+	Expect(ingressProg[0].Pref).To(Equal(ingProg[0].Pref))
+	Expect(ingressProg[0].Handle).To(Equal(ingProg[0].Handle))
+	Expect(egressProg[0].Pref).To(Equal(egrProg[0].Pref))
+	Expect(egressProg[0].Handle).To(Equal(egrProg[0].Handle))
+	Expect(ingProg[0].Pref).To(Equal(egrProg[0].Pref))
+	Expect(ingProg[0].Handle).To(Equal(egrProg[0].Handle))
+
+	// Verify that QoS state in map persists correctly after a workload endpoint update
+	qosValBytes1, err = qosMap.Get(qosKey1.AsBytes())
+	Expect(err).NotTo(HaveOccurred())
+	qosVal1 = qos.ValueFromBytes(qosValBytes1)
+	Expect(qosVal1.PacketRate()).To(Equal(int16(50)))
+	Expect(qosVal1.PacketBurst()).To(Equal(int16(100)))
+	Expect(qosVal1.PacketRateTokens()).To(Equal(int16(-1)))
+	Expect(qosVal1.PacketRateLastUpdate()).To(Equal(uint64(0)))
+
+	qosValBytes2, err = qosMap.Get(qosKey2.AsBytes())
+	Expect(err).NotTo(HaveOccurred())
+	qosVal2 = qos.ValueFromBytes(qosValBytes2)
+	Expect(qosVal2.PacketRate()).To(Equal(int16(200)))
+	Expect(qosVal2.PacketBurst()).To(Equal(int16(300)))
+	Expect(qosVal2.PacketRateTokens()).To(Equal(int16(-1)))
+	Expect(qosVal2.PacketRateLastUpdate()).To(Equal(uint64(0)))
 }
 
 // This test verifies if the tc program gets replaced
@@ -831,20 +893,22 @@ func TestRepeatedAttach(t *testing.T) {
 
 	_, err := tc.EnsureQdisc(ifaceName)
 	Expect(err).NotTo(HaveOccurred(), "failed to create qdisc")
-	res, err := ap.AttachProgram()
+	err = ap.AttachProgram()
 	Expect(err).NotTo(HaveOccurred(), "failed to attach preamble")
-	tcRes, ok := res.(tc.AttachResult)
-	Expect(ok).To(BeTrue())
-	prio := tcRes.Prio()
-	handle := tcRes.Handle()
+	ingressProg, err := tc.ListAttachedPrograms(ap.Iface, ap.Hook.String(), true)
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(ingressProg)).To(Equal(1))
 	for i := 0; i < 3; i++ {
-		res, err = ap.AttachProgram()
+		err = ap.AttachProgram()
 		Expect(err).NotTo(HaveOccurred(), fmt.Sprintf("failed to attach preamble : %d", i))
-		tcRes, ok = res.(tc.AttachResult)
-		Expect(ok).To(BeTrue())
-		Expect(tcRes.Prio()).To(Equal(prio))
-		Expect(tcRes.Handle()).To(Equal(handle))
 	}
+
+	ingProg, err := tc.ListAttachedPrograms(ap.Iface, ap.Hook.String(), true)
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(ingProg)).To(Equal(1))
+
+	Expect(ingProg[0].Pref).To(Equal(ingressProg[0].Pref))
+	Expect(ingProg[0].Handle).To(Equal(ingressProg[0].Handle))
 	// We have a BPF program attached to ingress hook and nothing on the egress hook.
 	// Now when there is a workload update, ingress program must be replaced and new program
 	// must be attached to egress.
@@ -883,18 +947,29 @@ func TestRepeatedAttach(t *testing.T) {
 	err = bpfEpMgr.CompleteDeferredWork()
 	Expect(err).NotTo(HaveOccurred())
 
-	// we update the state only after both ingress and egress qdiscs are same.
-	valid, newPrio, newHandle := bpfEpMgr.GetIfaceQDiscInfo("workloadep1")
-	Expect(valid).To(BeTrue())
-	Expect(newPrio).To(Equal(prio))
-	Expect(newHandle).To(Equal(handle))
+	ingProg, err = tc.ListAttachedPrograms(ap.Iface, hook.Ingress.String(), true)
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(ingProg)).To(Equal(1))
+
+	egrProg, err := tc.ListAttachedPrograms(ap.Iface, hook.Egress.String(), true)
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(egrProg)).To(Equal(1))
+
+	Expect(ingProg[0].Pref).To(Equal(ingressProg[0].Pref))
+	Expect(ingProg[0].Handle).To(Equal(ingressProg[0].Handle))
+	Expect(egrProg[0].Pref).To(Equal(ingressProg[0].Pref))
+	Expect(egrProg[0].Handle).To(Equal(ingressProg[0].Handle))
 }
 
 func TestCTLBAttachLegacy(t *testing.T) {
 	RegisterTestingT(t)
 
 	testCtlbAttachLegacy := func(v4, v6 bool) {
-		err := nat.InstallConnectTimeLoadBalancerLegacy(v4, v6, "", "debug", 60*time.Second, false)
+		bpfmaps, err := bpfmap.CreateBPFMaps(false)
+		Expect(err).NotTo(HaveOccurred())
+
+		commonMaps := bpfmaps.CommonMaps
+		err = nat.InstallConnectTimeLoadBalancerLegacy(v4, v6, "", "debug", 60*time.Second, false, commonMaps.CTLBProgramsMap)
 		Expect(err).NotTo(HaveOccurred())
 
 		checkPinPath := func(pinPath string, mustExist bool) {
@@ -964,7 +1039,11 @@ func TestCTLBAttachLegacy(t *testing.T) {
 func TestCTLBAttach(t *testing.T) {
 	RegisterTestingT(t)
 	testCtlbAttach := func(v4, v6 bool) {
-		err := nat.InstallConnectTimeLoadBalancer(v4, v6, "", "debug", 60*time.Second, false)
+		bpfmaps, err := bpfmap.CreateBPFMaps(false)
+		Expect(err).NotTo(HaveOccurred())
+
+		commonMaps := bpfmaps.CommonMaps
+		err = nat.InstallConnectTimeLoadBalancer(v4, v6, "", "debug", 60*time.Second, false, commonMaps.CTLBProgramsMap)
 		Expect(err).NotTo(HaveOccurred())
 
 		checkPinPath := func(pinPath string, mustExist bool) {
@@ -1045,6 +1124,208 @@ func TestCTLBAttach(t *testing.T) {
 	testCtlbAttach(true, true)
 }
 
+func TestAttachInterfaceRecreate(t *testing.T) {
+	RegisterTestingT(t)
+	bpfmaps, err := bpfmap.CreateBPFMaps(false)
+	Expect(err).NotTo(HaveOccurred())
+
+	loglevel := "off"
+	bpfEpMgr, err := newBPFTestEpMgr(
+		&linux.Config{
+			Hostname:              "uthost",
+			BPFLogLevel:           loglevel,
+			BPFDataIfacePattern:   regexp.MustCompile("^hostep[12]"),
+			VXLANMTU:              1000,
+			VXLANPort:             1234,
+			BPFNodePortDSREnabled: false,
+			RulesConfig: rules.Config{
+				EndpointToHostAction: "RETURN",
+			},
+			BPFExtToServiceConnmark: 0,
+			BPFPolicyDebugEnabled:   true,
+			BPFAttachType:           apiv3.BPFAttachOptionTCX,
+		},
+		bpfmaps,
+		regexp.MustCompile("^workloadep[0123]"),
+	)
+	Expect(err).NotTo(HaveOccurred())
+
+	workload0 := createVethName("workloadep0")
+	defer func() {
+		if workload0 != nil {
+			deleteLink(workload0)
+		}
+	}()
+
+	bpfEpMgr.OnUpdate(&proto.HostMetadataUpdate{Hostname: "uthost", Ipv4Addr: "1.2.3.4"})
+	bpfEpMgr.OnUpdate(linux.NewIfaceStateUpdate("workloadep0", ifacemonitor.StateUp, workload0.Attrs().Index))
+	bpfEpMgr.OnUpdate(linux.NewIfaceAddrsUpdate("workloadep0", "1.6.6.6"))
+	bpfEpMgr.OnUpdate(&proto.WorkloadEndpointUpdate{
+		Id: &proto.WorkloadEndpointID{
+			OrchestratorId: "k8s",
+			WorkloadId:     "workloadep0",
+			EndpointId:     "workloadep0",
+		},
+		Endpoint: &proto.WorkloadEndpoint{Name: "workloadep0"},
+	})
+	err = bpfEpMgr.CompleteDeferredWork()
+	Expect(err).NotTo(HaveOccurred())
+	_, err = os.Stat(bpfdefs.TcxPinDir + "/workloadep0_ingress")
+	Expect(err).NotTo(HaveOccurred())
+	_, err = os.Stat(bpfdefs.TcxPinDir + "/workloadep0_egress")
+	Expect(err).NotTo(HaveOccurred())
+
+	// Endpoint managed gets interface deleted but interface still exists.
+	// This can happen if the interface is deleted and recreated quickly.
+	// The BPF endpoint manager gets the update that interface is gone but
+	// the interface is still there. The pinned programs must remain.
+	bpfEpMgr.OnUpdate(linux.NewIfaceStateUpdate("workloadep0", ifacemonitor.StateNotPresent, workload0.Attrs().Index))
+	err = bpfEpMgr.CompleteDeferredWork()
+	Expect(err).NotTo(HaveOccurred())
+	_, err = os.Stat(bpfdefs.TcxPinDir + "/workloadep0_ingress")
+	Expect(err).NotTo(HaveOccurred())
+	_, err = os.Stat(bpfdefs.TcxPinDir + "/workloadep0_egress")
+	Expect(err).NotTo(HaveOccurred())
+
+	// Now simulate interface being deleted and recreated.
+	deleteLink(workload0)
+	workload0 = nil
+	workload0_new := createVethName("workloadep0")
+	defer func() {
+		if workload0_new != nil {
+			deleteLink(workload0_new)
+		}
+	}()
+	bpfEpMgr.OnUpdate(linux.NewIfaceStateUpdate("workloadep0", ifacemonitor.StateUp, workload0_new.Attrs().Index))
+	bpfEpMgr.OnUpdate(linux.NewIfaceAddrsUpdate("workloadep0", "1.6.6.6"))
+	err = bpfEpMgr.CompleteDeferredWork()
+	Expect(err).NotTo(HaveOccurred())
+	_, err = os.Stat(bpfdefs.TcxPinDir + "/workloadep0_ingress")
+	Expect(err).NotTo(HaveOccurred())
+	_, err = os.Stat(bpfdefs.TcxPinDir + "/workloadep0_egress")
+	Expect(err).NotTo(HaveOccurred())
+
+	// Interface is deleted. BPF endpoint manager gets the update.
+	// The pinned programs must be removed.
+	deleteLink(workload0_new)
+	workload0_new = nil
+
+	bpfEpMgr.OnUpdate(linux.NewIfaceStateUpdate("workloadep0", ifacemonitor.StateNotPresent, 0))
+	err = bpfEpMgr.CompleteDeferredWork()
+	Expect(err).NotTo(HaveOccurred())
+	_, err = os.Stat(bpfdefs.TcxPinDir + "/workloadep0_ingress")
+	Expect(err).To(HaveOccurred())
+	_, err = os.Stat(bpfdefs.TcxPinDir + "/workloadep0_egress")
+	Expect(err).To(HaveOccurred())
+}
+
+func TestAttachTcx(t *testing.T) {
+	RegisterTestingT(t)
+	bpfmaps, err := bpfmap.CreateBPFMaps(false)
+	Expect(err).NotTo(HaveOccurred())
+
+	loglevel := "off"
+	bpfEpMgr, err := newBPFTestEpMgr(
+		&linux.Config{
+			Hostname:              "uthost",
+			BPFLogLevel:           loglevel,
+			BPFDataIfacePattern:   regexp.MustCompile("^hostep[12]"),
+			VXLANMTU:              1000,
+			VXLANPort:             1234,
+			BPFNodePortDSREnabled: false,
+			RulesConfig: rules.Config{
+				EndpointToHostAction: "RETURN",
+			},
+			BPFExtToServiceConnmark: 0,
+			BPFPolicyDebugEnabled:   true,
+			BPFAttachType:           apiv3.BPFAttachOptionTCX,
+		},
+		bpfmaps,
+		regexp.MustCompile("^workloadep[0123]"),
+	)
+	Expect(err).NotTo(HaveOccurred())
+
+	workload0 := createVethName("workloadep0")
+	defer deleteLink(workload0)
+
+	bpfEpMgr.OnUpdate(&proto.HostMetadataUpdate{Hostname: "uthost", Ipv4Addr: "1.2.3.4"})
+	bpfEpMgr.OnUpdate(linux.NewIfaceStateUpdate("workloadep0", ifacemonitor.StateUp, workload0.Attrs().Index))
+	bpfEpMgr.OnUpdate(linux.NewIfaceAddrsUpdate("workloadep0", "1.6.6.6"))
+	bpfEpMgr.OnUpdate(&proto.WorkloadEndpointUpdate{
+		Id: &proto.WorkloadEndpointID{
+			OrchestratorId: "k8s",
+			WorkloadId:     "workloadep0",
+			EndpointId:     "workloadep0",
+		},
+		Endpoint: &proto.WorkloadEndpoint{Name: "workloadep0"},
+	})
+	err = bpfEpMgr.CompleteDeferredWork()
+	Expect(err).NotTo(HaveOccurred())
+	// Ensure there is no qdisc.
+	hasQdisc, err := tc.HasQdisc("workloadep0")
+	Expect(err).NotTo(HaveOccurred())
+	Expect(hasQdisc).To(BeFalse())
+	// Check if there are no tc programs.
+	progs, err := tc.ListAttachedPrograms("workloadep0", hook.Ingress.String(), true)
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(progs)).To(Equal(0))
+	progs, err = tc.ListAttachedPrograms("workloadep0", hook.Egress.String(), true)
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(progs)).To(Equal(0))
+
+	_, err = os.Stat(bpfdefs.TcxPinDir + "/workloadep0_ingress")
+	Expect(err).NotTo(HaveOccurred())
+	_, err = os.Stat(bpfdefs.TcxPinDir + "/workloadep0_egress")
+	Expect(err).NotTo(HaveOccurred())
+	tcxProgs, err := tc.ListAttachedTcxPrograms("workloadep0", "ingress")
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(tcxProgs)).To(Equal(1))
+	// Now attach Tc program.
+	ap := &tc.AttachPoint{
+		AttachPoint: bpf.AttachPoint{
+			Iface: "workloadep0",
+			Hook:  hook.Ingress,
+		},
+		HostIPv4:   net.IPv4(1, 2, 3, 4),
+		IntfIPv4:   net.IPv4(1, 6, 6, 6),
+		AttachType: apiv3.BPFAttachOptionTC,
+	}
+
+	_, err = tc.EnsureQdisc("workloadep0")
+	Expect(err).NotTo(HaveOccurred())
+	err = ap.AttachProgram()
+	Expect(err).NotTo(HaveOccurred())
+	progs, err = tc.ListAttachedPrograms("workloadep0", hook.Ingress.String(), true)
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(progs)).To(Equal(1))
+	_, err = os.Stat(bpfdefs.TcxPinDir + "/workloadep0_ingress")
+	Expect(err).To(HaveOccurred())
+	tcxProgs, err = tc.ListAttachedTcxPrograms("workloadep0", "ingress")
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(tcxProgs)).To(Equal(0))
+	// Now attach TCx again
+	bpfEpMgr.OnUpdate(&proto.WorkloadEndpointUpdate{
+		Id: &proto.WorkloadEndpointID{
+			OrchestratorId: "k8s",
+			WorkloadId:     "workloadep0",
+			EndpointId:     "workloadep0",
+		},
+		Endpoint: &proto.WorkloadEndpoint{Name: "workloadep0"},
+	})
+	err = bpfEpMgr.CompleteDeferredWork()
+	Expect(err).NotTo(HaveOccurred())
+	hasQdisc, err = tc.HasQdisc("workloadep0")
+	Expect(err).NotTo(HaveOccurred())
+	// switching from tcx to tc removes the qdisc.
+	Expect(hasQdisc).To(BeFalse())
+	progs, err = tc.ListAttachedPrograms("workloadep0", hook.Ingress.String(), true)
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(progs)).To(Equal(0))
+	tcxProgs, err = tc.ListAttachedTcxPrograms("workloadep0", "ingress")
+	Expect(err).NotTo(HaveOccurred())
+	Expect(len(tcxProgs)).To(Equal(1))
+}
+
 func TestLogFilters(t *testing.T) {
 	RegisterTestingT(t)
 
@@ -1156,6 +1437,17 @@ func jumpMapDump(m maps.Map) map[int]int {
 	return jumpMap
 }
 
+func qosMapDump(m maps.Map) qos.MapMem {
+	qosMap := make(qos.MapMem)
+	qosMapIter := qos.MapMemIter(qosMap)
+	_ = m.Iter(func(k, v []byte) maps.IteratorAction {
+		qosMapIter(k, v)
+		return maps.IterNone
+	})
+
+	return qosMap
+}
+
 func BenchmarkAttachProgram(b *testing.B) {
 	RegisterTestingT(b)
 
@@ -1180,7 +1472,7 @@ func BenchmarkAttachProgram(b *testing.B) {
 		IntfIPv4: net.IPv4(1, 1, 1, 1),
 	}
 
-	_, err = ap.AttachProgram()
+	err = ap.AttachProgram()
 	Expect(err).NotTo(HaveOccurred())
 
 	logLevel := log.GetLevel()
@@ -1190,7 +1482,7 @@ func BenchmarkAttachProgram(b *testing.B) {
 	b.StartTimer()
 
 	for n := 0; n < b.N; n++ {
-		_, err := ap.AttachProgram()
+		err := ap.AttachProgram()
 		if err != nil {
 			b.Fatalf("AttachProgram failed: %s", err)
 		}
