@@ -78,6 +78,7 @@ type TopologyOptions struct {
 	IPv6PoolUsages            []api.IPPoolAllowedUse
 	NeedNodeIP                bool
 	FlowLogSource             int
+	BPFProxyHealthzPort       int // zero means disable
 }
 
 // Calico containers created during topology creation.
@@ -309,6 +310,8 @@ func StartNNodeTopology(
 		typhaIP = tc.Typha.IP
 	}
 
+	opts.ExtraEnvVars["FELIX_BPFKUBEPROXYHEALTZPORT"] = fmt.Sprintf("%d", opts.BPFProxyHealthzPort)
+
 	tc.Felixes = make([]*Felix, n)
 	var wg sync.WaitGroup
 
